/*
 * Copyright 2005-2010 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */ 
package org.wamblee.support.persistence;

import javax.sql.DataSource;

import org.apache.commons.dbcp.ConnectionFactory;
import org.apache.commons.dbcp.DriverManagerConnectionFactory;
import org.apache.commons.dbcp.PoolableConnectionFactory;
import org.apache.commons.dbcp.PoolingDataSource;
import org.apache.commons.pool.impl.GenericObjectPool;

public abstract class AbstractDatabase implements Database {
    private static final int CONNECTION_POOL_SIZE = 16;

    private DataSource itsDataSource;

    private boolean started;

    protected AbstractDatabase() {
        started = false;
    }

    protected abstract void doStart();

    protected abstract void doStop();

    /**
     * This method must be called from the start method.
     */
    protected final void createDataSource() {
        GenericObjectPool connectionPool = new GenericObjectPool(null);
        connectionPool.setMaxActive(CONNECTION_POOL_SIZE);
        ConnectionFactory connectionFactory = new DriverManagerConnectionFactory(
            getJdbcUrl(), getUsername(), getPassword());
        // The following line must be kept in although it does not appear to be
        // used, the constructor regsiters the
        // constructed object at the connection pool.
        PoolableConnectionFactory poolableConnectionFactory = new PoolableConnectionFactory(
            connectionFactory, connectionPool, null, null, false, true);
        ingoredVariable(poolableConnectionFactory);
        itsDataSource = new PoolingDataSource(connectionPool);
    }
    
    private static void ingoredVariable(PoolableConnectionFactory aFactory) {
        // Empty
    }

    // / BELOW THIS LINE IS NOT OF INTEREST TO SUBCLASSES.

    public final DataSource start() {
        if (started) {
            throw new RuntimeException("Database already started");
        }
        started = true;
        doStart();
        return getDatasource();
    }

    public final void stop() {
        if (!started) {
            return; // nothing to do.
        }
        started = false;
        doStop();
    }

    private final DataSource getDatasource() {
        if (!started) {
            throw new RuntimeException("Database is not started!");
        }
        return itsDataSource;
    }

    protected String getProperty(String aName) {
        String value = System.getProperty(aName);
        if (value != null) {
            return value;
        }
        value = System.getenv(aName);
        if (value != null) {
            return value;
        }
        throw new RuntimeException("This class expects the '" + aName +
            "' property to be set");
    }

}
